﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EmailObfuscator.Contracts.Obfuscators;
using Orchard.ContentManagement;
using JetBrains.Annotations;
using Orchard.ContentManagement.Handlers;
using System.Text.RegularExpressions;
using System.Text;

namespace EmailObfuscator.Obfuscators
{
    [UsedImplicitly]
    public class EmailObfuscator : IObfuscator
    {
        #region fields

        /// <summary>
        /// the regular expression pattern for a email
        /// </summary>
        string emailPattern = @"(([^<>()[\]\\.,;:\s@\""]+"
                        + @"(\.[^<>()[\]\\.,;:\s@\""]+)*)|(\"".+\""))@"
                        + @"((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}"
                        + @"\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+"
                        + @"[a-zA-Z]{2,}))";

        #endregion

        #region constructor

        public EmailObfuscator()
        {

        }

        #endregion

        #region IObfuscator

        public string Obfuscate(string html)
        {
            return Regex.Replace(html, emailPattern, Munge);
        }

        #endregion

        /// <summary>
        /// php code transformed to C# for obfuscating email adresses
        /// would have been nicer if javascript could be placed in a view
        /// http://www.jottings.com/obfuscator/
        /// </summary>
        /// <param name="match">the match of the regex</param>
        /// <returns>obfuscated string</returns>
        private string Munge(Match match)
        {
            var adress = match.Value.ToLower();
            var coded = "";
            var unmixedkey = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789.@";
            var inprogresskey = unmixedkey;
            var mixedkey = "";
            var unshuffled = unmixedkey.Length;
            for (int i = 0; i < unmixedkey.Length; i++)
            {
                var ranpos = new Random().Next(0, unshuffled - 1);
                var nextchar = inprogresskey.ElementAt(ranpos);
                mixedkey += nextchar;
                var before = inprogresskey.Substring(0, ranpos);
                var after = inprogresskey.Substring(ranpos + 1, unshuffled - (ranpos + 1));
                inprogresskey = before + after;
                unshuffled -= 1;

            }
            var cipher = mixedkey;
            var shift = adress.Length;

            var stringbuilder = new StringBuilder();
            stringbuilder.Append("<script type=\"text/javascript\" language=\"javascript\">\n");
            stringbuilder.Append("<!--\n");
            stringbuilder.Append("// Email obfuscator script 2.1 by Tim Williams, University of Arizona\n");
            stringbuilder.Append("// Random encryption key feature by Andrew Moulden, Site Engineering Ltd\n");
            stringbuilder.Append("// This code is freeware provided these four comment lines remain intact\n");
            stringbuilder.Append("// A wizard to generate this code is at http://www.jottings.com/obfuscator/\n");

            for (int j = 0; j < adress.Length; j++)
            {
                if (!cipher.Contains(adress.ElementAt(j)))
                {
                    var chr = adress.ElementAt(j);
                    coded += adress.ElementAt(j);
                }
                else
                {
                    var chr = (cipher.IndexOf(adress.ElementAt(j)) + shift) % cipher.Length;
                    coded += cipher.ElementAt(chr);
                }
            }


            stringbuilder.Append("\n coded=\"" + coded + "\";\n");
            stringbuilder.Append(string.Format(" key =\"{0}\";\n" ,cipher));
            stringbuilder.Append(" shift=coded.length;\n" );
            stringbuilder.Append(" link = \"\";\n" );
            stringbuilder.Append(" for(i=0;i<coded.length;i++){\n");
            stringbuilder.Append("    if(key.indexOf(coded.charAt(i))==-1){\n" );
            stringbuilder.Append("      ltr=code.charAt(i)\n" );
            stringbuilder.Append("      link+= (ltr)\n" );
            stringbuilder.Append("     }\n" );
            stringbuilder.Append("    else{   \n" );
            stringbuilder.Append("       ltr=(key.indexOf(coded.charAt(i))-shift+key.length)%key.length \n" );
            stringbuilder.Append("       link +=(key.charAt(ltr))\n" );
            stringbuilder.Append("    }\n" );
            stringbuilder.Append("  }\n" );
            stringbuilder.Append("document.write(\"<a href='mailto:\"+link+\"'>\"+link+\"</a>\")\n" );
            stringbuilder.Append("\n" );
            stringbuilder.Append(" //-" + "->\n" );
            stringbuilder.Append("<" + "/script><noscript>N/A <" + "/noscript>");

            return stringbuilder.ToString();
        }

    }
}